"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _jsx = require("../../../jsx");

var defaultProps = {
  offsetX: 0,
  offsetY: 0,
  points: [],
  direct: 'tl',
  side: 6,
  autoAdjust: true
};
var defaultStyle = {
  container: {
    fill: '#1677FF',
    radius: 2,
    padding: [3, 5]
  },
  text: {
    fontSize: '22px',
    fill: '#fff'
  },
  arrow: {
    fill: '#1677FF'
  }
};

var _default = function _default(props, context) {
  var cfg = (0, _objectSpread2.default)((0, _objectSpread2.default)({}, defaultProps), props);
  var points = cfg.points,
      content = cfg.content,
      offsetX = cfg.offsetX,
      offsetY = cfg.offsetY,
      direct = cfg.direct,
      side = cfg.side,
      autoAdjust = cfg.autoAdjust,
      canvasWidth = cfg.canvasWidth,
      canvasHeight = cfg.canvasHeight,
      guideBBox = cfg.guideBBox,
      background = cfg.background,
      textStyle = cfg.textStyle;

  var _ref = points[0] || {},
      x = _ref.x,
      y = _ref.y;

  var _ref2 = guideBBox || {},
      guideWidth = _ref2.width,
      guideHeight = _ref2.height;

  var offsetXNum = context.px2hd(offsetX);
  var offsetYNum = context.px2hd(offsetY);
  var posX = x + (offsetXNum || 0);
  var posY = y + (offsetYNum || 0);

  var _getDirect = function _getDirect(point) {
    var newDirect = direct;
    var x = point.x,
        y = point.y;
    var vertical = newDirect[0];
    var horizontal = newDirect[1]; // adjust for vertical direction

    if (vertical === 't' && y - side - guideHeight < 0) {
      vertical = 'b';
    } else if (vertical === 'b' && y + side + guideHeight > canvasHeight) {
      vertical = 't';
    } // adjust for horizontal direction


    var diff = vertical === 'c' ? side : 0;

    if (horizontal === 'l' && x - diff - guideWidth < 0) {
      horizontal = 'r';
    } else if (horizontal === 'r' && x + diff + guideWidth > canvasWidth) {
      horizontal = 'l';
    } else if (horizontal === 'c') {
      if (guideWidth / 2 + x + diff > canvasWidth) {
        horizontal = 'l';
      } else if (x - guideWidth / 2 - diff < 0) {
        horizontal = 'r';
      }
    }

    newDirect = vertical + horizontal;
    return newDirect;
  };

  var _getArrowPoints = function _getArrowPoints(direct) {
    var arrowPoints = [];

    var _ref3 = guideBBox || {},
        minX = _ref3.minX,
        minY = _ref3.minY;

    if (direct === 'tl') {
      arrowPoints = [{
        x: minX,
        y: minY - 1
      }, {
        x: minX,
        y: minY + side
      }, {
        x: minX - side - 1,
        y: minY - 1
      }];
      posX -= guideWidth || 0;
      posY = posY - (guideHeight || 0) - side;
    } else if (direct === 'cl') {
      arrowPoints = [{
        x: minX - 1,
        y: minY - 1 - side / 2
      }, {
        x: minX - 1,
        y: minY + 1 + side / 2
      }, {
        x: minX + side,
        y: minY
      }];
      posX = posX - (guideWidth || 0) - side;
      posY -= guideHeight / 2 || 0;
    } else if (direct === 'bl') {
      arrowPoints = [{
        x: minX,
        y: -side + minY
      }, {
        x: minX - side - 1,
        y: minY + 1
      }, {
        x: minX,
        y: minY + 1
      }];
      posX = posX - (guideWidth || 0);
      posY += side;
    } else if (direct === 'bc') {
      arrowPoints = [{
        x: guideWidth / 2 + minX,
        y: -side + minY
      }, {
        x: (guideWidth - side) / 2 + minX - 1,
        y: minY + 1
      }, {
        x: (guideWidth + side) / 2 + minX + 1,
        y: minY + 1
      }];
    } else if (direct === 'br') {
      arrowPoints = [{
        x: minX,
        y: minY - side
      }, {
        x: minX,
        y: minY + 1
      }, {
        x: minX + side + 1,
        y: minY + 1
      }];
      posY += side;
    } else if (direct === 'cr') {
      arrowPoints = [{
        x: minX - side,
        y: minY
      }, {
        x: minX + 1,
        y: minY - 1 - side / 2
      }, {
        x: minX + 1,
        y: minY + 1 + side / 2
      }];
      posX += side;
      posY -= guideHeight / 2 || 0;
    } else if (direct === 'tr') {
      arrowPoints = [{
        x: minX,
        y: minY + side
      }, {
        x: minX,
        y: minY - 1
      }, {
        x: side + minX + 1,
        y: minY - 1
      }];
      posY = posY - (guideHeight || 0) - side;
    } else if (direct === 'tc') {
      arrowPoints = [{
        x: minX - 1 - side / 2,
        y: minY - 1
      }, {
        x: minX + 1 + side / 2,
        y: minY - 1
      }, {
        x: minX,
        y: minY + side
      }];
      posX -= guideWidth / 2 || 0;
      posY = posY - (guideHeight || 0) - side;
    }

    return arrowPoints;
  };

  var dr = autoAdjust ? _getDirect(points[0]) : direct;

  var arrowPoints = _getArrowPoints(dr);

  return (0, _jsx.jsx)("group", {
    attrs: (0, _objectSpread2.default)({
      fill: defaultStyle.container.fill,
      radius: defaultStyle.container.radius
    }, background),
    style: (0, _objectSpread2.default)({
      left: posX,
      top: posY,
      padding: defaultStyle.container.padding
    }, background)
  }, (0, _jsx.jsx)("text", {
    attrs: (0, _objectSpread2.default)({
      text: content,
      fontSize: defaultStyle.text.fontSize,
      fill: defaultStyle.text.fill
    }, textStyle)
  }), guideBBox && (0, _jsx.jsx)("polygon", {
    attrs: {
      points: arrowPoints,
      fill: (background === null || background === void 0 ? void 0 : background.fill) || defaultStyle.arrow.fill
    }
  }));
};

exports.default = _default;