"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _classCallCheck2 = _interopRequireDefault(require("@babel/runtime/helpers/classCallCheck"));

var _createClass2 = _interopRequireDefault(require("@babel/runtime/helpers/createClass"));

var _assertThisInitialized2 = _interopRequireDefault(require("@babel/runtime/helpers/assertThisInitialized"));

var _inherits2 = _interopRequireDefault(require("@babel/runtime/helpers/inherits"));

var _createSuper2 = _interopRequireDefault(require("@babel/runtime/helpers/createSuper"));

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread2"));

var _component = _interopRequireDefault(require("../../base/component"));

var _zoomUtil = require("./zoomUtil");

var MIN_COUNT = 10;

function cloneScale(scale, scaleConfig) {
  // @ts-ignore
  return new scale.constructor((0, _objectSpread2.default)((0, _objectSpread2.default)({}, scale.__cfg__), scaleConfig));
} // 缩放


var Zoom = /*#__PURE__*/function (_Component) {
  (0, _inherits2.default)(Zoom, _Component);

  var _super = (0, _createSuper2.default)(Zoom);

  function Zoom(props) {
    var _this;

    (0, _classCallCheck2.default)(this, Zoom);
    _this = _super.call(this, props);

    _this.onStart = function () {
      var _assertThisInitialize = (0, _assertThisInitialized2.default)(_this),
          state = _assertThisInitialize.state;

      var range = state.range;
      _this.startRange = range;
    };

    _this.onPan = function (ev) {
      var _this$props$mode = _this.props.mode,
          mode = _this$props$mode === void 0 ? 'x' : _this$props$mode;

      if (mode === 'x') {
        _this._doXPan(ev);

        return;
      }

      if (mode === 'y') {
        _this._doYPan(ev);

        return;
      }
    };

    _this.onPinch = function (ev) {
      var _this$props$mode2 = _this.props.mode,
          mode = _this$props$mode2 === void 0 ? 'x' : _this$props$mode2;

      if (mode === 'x') {
        _this._doXPinch(ev);

        return;
      }

      if (mode === 'y') {
        _this._doYPinch(ev);

        return;
      }
    };

    _this.onEnd = function () {
      _this.startRange = null;
    };

    var _props$range = props.range,
        range = _props$range === void 0 ? [0, 1] : _props$range;
    _this.state = {
      range: range
    };
    return _this;
  }

  (0, _createClass2.default)(Zoom, [{
    key: "didMount",
    value: function didMount() {
      this._bindEvents();
    }
  }, {
    key: "willMount",
    value: function willMount() {
      var props = this.props;
      var range = props.range;

      var scale = this._getScale();

      var values = scale.values;
      this.scale = scale;
      this.originScale = cloneScale(scale); // 图表上最少显示 MIN_COUNT 个数据

      this.minScale = MIN_COUNT / values.length;
      this.updateRange(range);
    }
  }, {
    key: "didUnmount",
    value: function didUnmount() {
      this._clearEvents();
    }
  }, {
    key: "_doXPan",
    value: function _doXPan(ev) {
      var direction = ev.direction,
          deltaX = ev.deltaX;

      if (direction === 'up' || direction === 'down') {
        return;
      }

      ev.preventDefault && ev.preventDefault();
      var props = this.props;
      var coord = props.coord;
      var coordWidth = coord.width;
      var ratio = deltaX / coordWidth;

      this._doPan(ratio);
    }
  }, {
    key: "_doYPan",
    value: function _doYPan(ev) {
      var direction = ev.direction,
          deltaY = ev.deltaY;

      if (direction === 'left' || direction === 'right') {
        return;
      }

      ev.preventDefault && ev.preventDefault();
      var props = this.props;
      var coord = props.coord;
      var coordHeight = coord.height;
      var ratio = -deltaY / coordHeight;

      this._doPan(ratio);
    }
  }, {
    key: "_doPan",
    value: function _doPan(ratio) {
      var startRange = this.startRange;

      var _startRange = (0, _slicedToArray2.default)(startRange, 2),
          start = _startRange[0],
          end = _startRange[1];

      var rangeLen = end - start;
      var rangeOffset = rangeLen * ratio;
      var newStart = start - rangeOffset;
      var newEnd = end - rangeOffset;
      this.updateRange([newStart, newEnd]);
    }
  }, {
    key: "_doXPinch",
    value: function _doXPinch(ev) {
      ev.preventDefault && ev.preventDefault();
      var zoom = ev.zoom,
          center = ev.center;
      var props = this.props;
      var coord = props.coord;
      var coordWidth = coord.width,
          left = coord.left,
          right = coord.right;
      var leftLen = Math.abs(center.x - left);
      var rightLen = Math.abs(right - center.x); // 计算左右缩放的比例

      var leftZoom = leftLen / coordWidth;
      var rightZoom = rightLen / coordWidth;

      this._doPinch(leftZoom, rightZoom, zoom);
    }
  }, {
    key: "_doYPinch",
    value: function _doYPinch(ev) {
      ev.preventDefault && ev.preventDefault();
      var zoom = ev.zoom,
          center = ev.center;
      var props = this.props;
      var coord = props.coord;
      var coordHeight = coord.height,
          top = coord.top,
          bottom = coord.bottom;
      var topLen = Math.abs(center.x - top);
      var bottomLen = Math.abs(bottom - center.x); // 计算左右缩放的比例

      var topZoom = topLen / coordHeight;
      var bottomZoom = bottomLen / coordHeight;

      this._doPinch(topZoom, bottomZoom, zoom);
    }
  }, {
    key: "_doPinch",
    value: function _doPinch(startRatio, endRatio, zoom) {
      var startRange = this.startRange,
          minScale = this.minScale;

      var _startRange2 = (0, _slicedToArray2.default)(startRange, 2),
          start = _startRange2[0],
          end = _startRange2[1];

      var zoomOffset = 1 - zoom;
      var rangeLen = end - start;
      var rangeOffset = rangeLen * zoomOffset;
      var startOffset = rangeOffset * startRatio;
      var endOffset = rangeOffset * endRatio;
      var newStart = Math.max(0, start - startOffset);
      var newEnd = Math.min(1, end + endOffset);
      var newRange = [newStart, newEnd]; // 如果已经到了最小比例，则不能再继续再放大

      if (newEnd - newStart < minScale) {
        return;
      }

      this.updateRange(newRange);
    }
  }, {
    key: "updateRange",
    value: function updateRange(range) {
      if (!range) return;

      var _range = (0, _slicedToArray2.default)(range, 2),
          start = _range[0],
          end = _range[1];

      var rangeLength = end - start; // 处理边界值

      var newRange;

      if (start < 0) {
        newRange = [0, rangeLength];
      } else if (end > 1) {
        newRange = [1 - rangeLength, 1];
      } else {
        newRange = range;
      }

      var props = this.props,
          scale = this.scale,
          originScale = this.originScale; // 更新主 scale

      (0, _zoomUtil.updateRange)(scale, originScale, newRange);
      var chart = props.chart,
          data = props.data,
          autoFit = props.autoFit;

      if (autoFit) {
        var followScale = this._getFollowScales();

        this.updateFollow(followScale, scale, data);
      }

      this.setState({
        range: newRange
      }); // 手势变化不执行动画

      var animate = chart.animate;
      chart.setAnimate(false);
      chart.forceUpdate(function () {
        chart.setAnimate(animate);
      });
    }
  }, {
    key: "updateFollow",
    value: function updateFollow(scales, mainScale, data) {
      (0, _zoomUtil.updateFollow)(scales, mainScale, data);
    }
  }, {
    key: "_getScale",
    value: function _getScale() {
      var _this$props = this.props,
          _this$props$mode3 = _this$props.mode,
          mode = _this$props$mode3 === void 0 ? 'x' : _this$props$mode3,
          coord = _this$props.coord,
          chart = _this$props.chart;

      if (mode === 'x') {
        return coord.transposed ? chart.getYScales()[0] : chart.getXScales()[0];
      }

      if (mode === 'y') {
        return coord.transposed ? chart.getXScales()[0] : chart.getYScales()[0];
      }
    }
  }, {
    key: "_getFollowScales",
    value: function _getFollowScales() {
      var _this$props2 = this.props,
          _this$props2$mode = _this$props2.mode,
          mode = _this$props2$mode === void 0 ? 'x' : _this$props2$mode,
          coord = _this$props2.coord,
          chart = _this$props2.chart;

      if (mode === 'x') {
        return coord.transposed ? chart.getXScales() : chart.getYScales();
      }

      if (mode === 'y') {
        return coord.transposed ? chart.getYScales() : chart.getXScales();
      }
    }
  }, {
    key: "_bindEvents",
    value: function _bindEvents() {
      var context = this.context,
          props = this.props;
      var canvas = context.canvas;
      var pan = props.pan,
          pinch = props.pinch; // 统一绑定事件

      if (pan !== false) {
        canvas.on('panstart', this.onStart);
        canvas.on('pan', this.onPan);
        canvas.on('panend', this.onEnd);
      }

      if (pinch !== false) {
        canvas.on('pinchstart', this.onStart);
        canvas.on('pinch', this.onPinch);
        canvas.on('pinchend', this.onEnd);
      }
    }
  }, {
    key: "_clearEvents",
    value: function _clearEvents() {
      var context = this.context,
          props = this.props;
      var canvas = context.canvas;
      var pan = props.pan,
          pinch = props.pinch; // 统一解绑事件

      if (pan !== false) {
        canvas.off('panstart', this.onStart);
        canvas.off('pan', this.onPan);
        canvas.off('panend', this.onEnd);
      }

      if (pinch !== false) {
        canvas.off('pinchstart', this.onStart);
        canvas.off('pinch', this.onPinch);
        canvas.off('pinchend', this.onEnd);
      }
    }
  }]);
  return Zoom;
}(_component.default);

var _default = Zoom;
exports.default = _default;